"""Constants for the pyaffalddk integration."""
from __future__ import annotations

NON_SUPPORTED_ITEMS = [
    "Asbest",
    "Beholderservice",
    "Bestillerordning",
    "Henteordning for grene",
    "Trærødder og stammer",
    "Ingen tømningsdato fundet!",
    "Skal tilmeldes",
    "Bestil afhentning",
    "Farligt affald (skal bestilles)",
    "Storskrald (skal bestilles)",
    "Haveaffald (skal bestilles)",
    "Beholderudbringning",
]

SUPPORTED_ITEMS = {
    "restaffaldmadaffald": [
        "Restaffald-Madaffald",
        "Rest/mad",
        "Restaffald",
        "Rest - Mad",
        "Rest-/Madaffald",
        "Mad- og restaffald",
        "Mad/rest",
        "Rest-/madaffald",
        "Mad/Rest",
        "Mad og restaffald",
        "Mad- og restaffald",
        "Mad-/ og restaffald",
        "Rest / Mad",
        "Rest Mad",
        "Energispand - Obligatorisk min. 1 spand",
        "Rest/madaffald",
        "Energibeholder (mad/rest)",
        "Rest- og Madaffald",
    ],
    "madaffald": [
        "Madaffald",
        "Bioposer",
    ],
    "batterier": ["Batterier"],
    "dagrenovation": ["Dagrenovation"],
    "elektronik": ["Elektronik"],
    "glas": [
        "Industri Genbrugeligt",
        "Glas",
    ],
    "metalglas": [
        "Glas og metal",
        "Metal-Glas",
        "Glas/Metal",
        "Metal og Glas",
        "Glas / Metal",],
    "pappi": [
        "Plast MDK og papir",
        "PAPPI",
        "Plast/MD-karton/PP",
        "Plast/Papir",
        "Papir/Plast og kartoner",
        "Plast og Mad- & drikkekartoner/Papir",
        "Plast, Mad- og drikkekartoner, Småt Pap og Papir",
        "Papir & Pap / Plast & MDK",
    ],
    "farligtaffald": [
        "Farligt affald",
        "Miljøkasser",
        "Farligt Affald, Rød boks",
        "Genbrugsbilen",
    ],
    "farligtaffaldmiljoboks": [
        "Farligt affald/Miljøboks",
        "Papir og glas/dåser",
        "Miljøboks",
    ],
    "flis": [
        "Flis",
        "Flishugning",
    ],
    "genbrug": [
        "Genbrug",
        "Genbrugsøer",
        "Genanvendeligt",
        "Genbrug 240 L",
        "Genbrugshal (1 stk.)",
    ],
    "jern": [
        "Jern",
        "Metal",
    ],
    "juletrae": [
        "Juletræ",
    ],
    "papir": [
        "Papir",
        "Papir 660 L",
        "Papir // Tekstilaffald",
    ],
    "papirglasdaaser": [
        "Papir og glas/dåser",
    ],
    "papirmetal": [
        "Papir & Metal",
        "Papir/metal",
        "Metal/papir",
        "Papir og Metal",
        "Pap & metal",
    ],
    "pap": [
        "Pap",
    ],
    "plastmetal": [
        "Plast & Metal",
        "Plast, metal & MDK 660L",
        "Plast/ Metal",
        "Plast/MDK/Metal",
        "Plast-metal",
        "Plast/Metal",
        "Plast, Metal, MDK",
    ],
    "restaffald": [
        "Restaffald // Batterier og Småt Elektronik",
        "Industri Restaffald",
    ],
    "storskrald": [
        "Storskrald",
        "Stort elektronik",
        "Storskrald og genbrug",
        "Pap og Storskrald",
        "Stort indbo og Indendørs træ",
    ],
    "storskraldogtekstilaffald": [
        "Storskrald og tekstilaffald",
    ],
    "haveaffald": [
        "Haveaffald, flishugning",
        "Haveaffald",
        "Trærødder og stammer",
        "Haveaffald - Frivilligt",
        "Haveaffald 140 L",
        "Frivillig Haveaffald tømmes fra mandag til onsdag",
    ],
    "papirglas": [
        "Papir og pap/Glas",
        "Glas-papir",
        "Papir, Pap/Glas",
        "Papir & glas",
    ],
    "plastmadkarton": [
        "Plast/MDK",
        "Plast & mad- og drikkekartoner",
        "Plast/ Mad- og drikkekartoner",
        "Plast og MDK",
    ],
    "pappapirglasmetal": [
        "Pap, papir, tekstil & metal og glas",
        "4delt beholder",
        "Ressourcebeholder (pap/papir og glas/metal)",
        "Papir/pap og glas/metal",
    ],
    "plastmetalmadmdk": [
        "Plast/Metal/Mad- & drikkekartoner",
        "Plast, mad- og drikkekartoner og metal",
    ],
    "plastmdkglasmetal": [
        "Plast og drikkekarton – Glas og metal",
    ],
    "pappapir": [
        "Pap/Papir",
        "Papir/Pap",
        "Papir og Pap",
        "Papir, pap og tekstilaffald",
    ],
    "tekstil": [
        "Tekstiler",
        "Standplads",
        "Tekstilaffald",
        "Miljøkasse/tekstiler",
        "Tekstil",
        "Tekstilposer (1 stk.)",
    ],
    "glasplast": [
        "Glas/Plast og MDK",
    ],
    "plastmetalpapir": [
        "Plast/Metal/Papir ",
        "Genbrugsspand - Obligatorisk min. 1 spand",
    ],
    "plast": ["Plast - Obligatorisk min. 1 spand", "Plast"],
    "restplast": [
        "Rest + plast/MDK",
    ],
}

NON_MATERIAL_LIST = [
    "Pap bundtet, havebolig (1 stk.)",
]

MATERIAL_LIST = {
    "restaffaldmadaffald": [
        "Mad- og restaffald",
        "*190 l rød, 14 dags tømning (1 stk.)",
    ],
    "batterier": [
        "Batterier i pose på låg (1 stk.)",
    ],
    "dagrenovation": [
        "240 l. container 1 x ugentligt (12 stk.)",
        "*140 l grøn (1 stk.)",
    ],
    "elektronik": [
        "Småt elektronik i pose på låg (1 stk.)",
    ],
    "glas": [
        "240 L glas (1 stk.)",
        "Glas, 660 liter (2 stk.)",
        "400 liter - Glas - Tømmes hver 8. uge (1 stk.)",
        "Glas",
        "Cube 1500 L (Flasker) (1 stk.)",
    ],
    "metalglas": [
        "Glas/metal (1 stk.)",
        "240 l Glas/metal Egenløsning (1 stk.)",
        "Metal/Glas 240 l - 2-kammer (1 stk.)",
        "Glas/metal, afstand over 5 meter (1 stk.)",
        "240 L - Metal og Glas (1 stk.)",
        "240 l glas/metal - 4 tømninger (1 stk.)",
        "240 liter - Metal/Glas - tømmes hver 8. uge (1 stk.)",
        "Metal/glas 240 l (hver 12. uge) (1 stk.)",
        "240 L. Glas/Metal - 4 tømninger (1 stk.)",
        "240 l Glas/metal Egenløsning (Privat) (1 stk.)",
        "Glas & Metal",
    ],
    "pappi": [
        "Plast/papir (1 stk.)",
        "370 l Plast+MDK/Papir - Egenløsning (1 stk.)",
        "240 l Plast+MDK/Papir - Egenløsning (1 stk.)",
        "Papir/Plast og MDK 240 l - 2-kammer (1 stk.)",
        "Plast/papir, afstand over 5 meter (1 stk.)",
        "240 L - Plast og Papir (1 stk.)",
        "240L genbrug (1 stk.)",
        "240 l PMDK/PP 3 ugers tømning (1 stk.)",
        "240 liter - Plast, MDK, papir, pap – tømmes hver 2. uge (1 stk.)",
        "PP/MDK-plast 240 l (hver 2. uge) (1 stk.)",
        "240 L. PMDK/PP 3 ugers tømning (1 stk.)",
        "370 l Plast+MDK/Papir - Egenløsning (Privat) (1 stk.)",
        "240 l Plast+MDK/Papir - Egenløsning (Privat) (1 stk.)",
        "Papir & småt pap",
    ],
    "farligtaffald": [
        "",
    ],
    "farligtaffaldmiljoboks": [
        "Miljøboks",
        "Miljøkasse (1 stk.)",
        "Miljøskab (1 stk.)",
        "Rød kasse henteordning (1 stk.)",
        "*Miljøkasse (1 stk.)",
    ],
    "flis": [""],
    "jern": [
        "240 L metal (1 stk.)",
        "Metal, 660 liter (1 stk.)",
        "240 liter - Metal - Tømmes hver 8. uge (1 stk.)",
        "Metal",
    ],
    "genbrug": [
        "Tekstiler",
        "Genbrug",
        "Genbrugsøer",
        "Genanvendeligt",
        "Genbrug 240 L",
        "Genbrugshal (1 stk.)",
    ],
    "papir": [
        "660 L papir (1 stk.)",
        "Papir, 660 liter (3 stk.)",
        "Papir",
    ],
    "papirmetal": [
        "Metal og papir 240 L (ejer.kommune) (1 stk.)",
    ],
    "pap": [
        "Pap 240 L (villa) (1 stk.)",
        "240 l Pap Egenløsning (1 stk.)",
        "Pap 240 L (1 stk.)",
        "Pap 240 L (ejer:kommune) (h) (1 stk.)",
        "Pap - 240 l.  (1 stk.)",
        "Indsamling af pap (løst og beholder) (1 stk.)",
        "240 L - Pap - Skel (1 stk.)",
        "Beholder til pap (1 stk.)",
        "770 L pap (1 stk.)",
        "Pap, 660 liter (3 stk.)",
        "400 liter - Pap og papir - Tømmes hver 2. uge (1 stk.)",
        "Pap",
        "240 l Pap Egenløsning (Privat) (1 stk.)",
    ],
    "plastmetal": [
        "Plast, småt metal & MDK - 240 l. (1 stk.)",
    ],
    "plastmdkglasmetal": [
        "240 L todelt genbrugsspand hver 4. uge (1 stk.)",
        "240L plast, MDK/metal, glas gl. (1 stk.)",
        "Genbrugsbeholder, 190 ltr (1 stk.)",
        "Genbrugsspand, 240L (2-delt) (1 stk.)",
    ],
    "plast": [
        "Plast og MDK 240 L (ejer.kommune) (1 stk.)",
        "Plast, 660 liter (2 stk.)",
        "Plastaffald henteordning (1 stk.)",
        "Plast",
    ],
    "storskrald": [
        "Storskrald - fortovsindsamling (1 stk.)",
        "Storskrald (1 stk.)",
        "Storskrald enfamiliehus (1 stk.)",
        "Storskrald",
        "Stort elektronik",
        "Storskrald og genbrug",
        "Pap og Storskrald",
        "Storskrald distrikt 8 (1 stk.)",
        "Møbler/indbo  (1 stk.)",
        "Elektronik, stort metal, keramik og porcelæn (1 stk.)",
        "Småt brændbart (1 stk.)",
    ],
    "storskraldogtekstilaffald": [""],
    "haveaffald": [
        "Gen-Skel 0-2 meter (1 stk.)",
        "Haveaffald henteordning (1 stk.)",
    ],
    "papirglas": [
        "Papir/glas - 240 l. fortovsindsamling (1 stk.)",
    ],
    "plastmadkarton": [
        "Genbrugsbeholder PMDK/MG-240L/3uge (1 stk.)",
        "Genbrug henteordning (1 stk.)",
        "660 L plast, mad- og drikkekartoner (1 stk.)",
        "240 l genbrug - blåt låg (1 stk.)",
        "660 liter - Plast/MDK - Tømmes hver 2. uge (1 stk.)",
        "Plast og mad- & drikkekartoner",
        "Mad- og drikkekartoner",
    ],
    "papirglasdaaser": [
        "240 L 2-delt Papir/glas-dåser en-familie (1 stk.)",
        "*240 L 2-delt Papir/glas-dåser en-familie (1 stk.)",
    ],
    "pappapirglasmetal": [
        "240L metal, glas, plast/papir gl. (1 stk.)",
        "240 l genbrug 2-kammer (2023) (1 stk.)",
        "4-kammer (240 l) (1 stk.)",
        "240 L 2-delt Papir/glas-dåser en-familie (1 stk.)",
        "240 l delt genbrug - grønt låg (1 stk.)",
        "Glas & metal og papir & småt pap",
    ],
    "plastmetalmadmdk": [""],
    "pappapir": [
        "Genbrugsbeholder PP-240L/6uge (1 stk.)",
        "240L papir, pap (1 stk.)",
    ],
    "tekstil": [
        "Tekstil pose (1 stk.)",
        "Tekstilpose tømning (1 stk.)",
        "Tekstilaffald (1 stk.)",
        "Tekstilpose 25 L (1 stk.)",
        "Tekstilaffald (i lukket pose) (1 stk.)",
    ],
    "glasplast": [""],
    "plastmetalpapir": [
        "*140 l grøn, 14 dags tømning (1 stk.)",
    ],
    "papirglasmetalplast": [
        "4-kammer (370 l) (1 stk.)",
        "Pap og papir/metal, glas og hård plast - 240 L (1 stk.)",
        "240 l genbrug låg i låg (1 stk.)",
        "Pap og papir/metal, glas og hård plast - 240 L (EX) (1 stk.)",
    ],
    "madaffald": [
        "140 L madaffald (1 stk.)",
        "*190 l rød (1 stk.)",
        "Madaffald",
    ],
    "restaffald": [
        "9505: 770 L restaffald uge (2 stk.)",
        "Restaffald",
    ],
    "juletrae": [
        "Juletræer (1 stk.)",
        "Juletræ",
    ],
}

ICON_LIST = {
    "batterier": "mdi:battery",
    "elektronik": "mdi:power-plug",
    "restaffaldmadaffald": "mdi:trash-can",
    "dagrenovation": "mdi:trash-can",
    "glas": "mdi:bottle-soda",
    "metalglas": "mdi:glass-fragile",
    "pappi": "mdi:trash-can",
    "farligtaffald": "mdi:recycle",
    "farligtaffaldmiljoboks": "mdi:recycle",
    "flis": "mdi:tree",
    "genbrug": "mdi:recycle",
    "jern": "mdi:bucket",
    "papir": "mdi:file",
    "papirmetal": "mdi:delete-empty",
    "pap": "mdi:note",
    "plastmetal": "mdi:trash-can-outline",
    "storskrald": "mdi:table-furniture",
    "storskraldogtekstilaffald": "mdi:table-furniture",
    "haveaffald": "mdi:leaf",
    "papirglas": "mdi:greenhouse",
    "papirglasmetalplast": "mdi:trash-can",
    "plastmadkarton": "mdi:trash-can",
    "papirglasdaaser": "mdi:trash-can",
    "pappapirglasmetal": "mdi:trash-can",
    "plastmdkglasmetal": "mdi:trash-can",
    "plastmetalmadmdk": "mdi:trash-can",
    "pappapir": "mdi:file",
    "tekstil": "mdi:recycle",
    "glasplast": "mdi:trash-can",
    "plastmetalpapir": "mdi:trash-can",
    "plast": "mdi:trash-can",
    "madaffald": "mdi:trash-can",
    "restaffald": "mdi:trash-can",
    "restplast": "mdi:trash-can",
    "juletrae": "mdi:pine-tree",
    "metal": "mdi:anvil",
}

NAME_LIST = {
    "batterier": "Batterier",
    "bioposer": "Bioposer",
    "elektronik": "Elektronik",
    "dagrenovation": "Dagrenovation",
    "farligtaffald": "Farligt affald",
    "farligtaffaldmiljoboks": "Farligt affald & Miljøboks",
    "flis": "Flis",
    "genbrug": "Genbrug",
    "glas": "Glas",
    "glasplast": "Glas, Plast & Madkartoner",
    "haveaffald": "Haveaffald",
    "jern": "Metal",
    "juletrae": "Juletræer",
    "madaffald": "Madaffald",
    "metal": "Metal",
    "metalglas": "Metal & Glas",
    "pap": "Pap",
    "papir": "Papir",
    "papirglas": "Papir, Pap & Glas",
    "papirglasdaaser": "Papir, Glas & Dåser",
    "papirglasmetalplast": "Papir, Glas, Metal & Plast",
    "papirmetal": "Papir & Metal",
    "pappapir": "Pap & Papir",
    "pappapirglasmetal": "Pap, Papir, Glas & Metal",
    "pappi": "Papir & Plast",
    "plast": "Plast",
    "plastmadkarton": "Plast & Madkarton",
    "plastmdkglasmetal": "Plast, Madkarton, Glas & Metal",
    "plastmetal": "Plast & Metal",
    "plastmetalmadmdk": "Plast, Metal, Mad & Drikkekartoner",
    "plastmetalpapir": "Plast, Metal & Papir",
    "restaffald": "Restaffald",
    "restaffaldmadaffald": "Rest & Madaffald",
    "restplast": "Restaffald & Plast/Madkartoner",
    "storskrald": "Storskrald",
    "storskraldogtekstilaffald": "Storskrald & Tekstilaffald",
    "tekstil": "Tekstilaffald",
}

NAME_ARRAY = list(NAME_LIST.keys())

MUNICIPALITIES_LIST = {
    "Aabenraa": ["aabenraa", "1"],
    "Aalborg": ["aalborg", "1"],
    "Aarhus": ["aarhus", "3"],
    "Albertslund": ["albertslund", "1"],
    "Allerød": ["allerod", "1"],
    "Billund": ["billund", "1"],
    "Bornholm": ["bofa", "1"],
    "Brøndby": ["brondby", "1"],
    "Brønderslev": ["bronderslev", "1"],
    "Dragør": ["dragoer", "1"],
    "Egedal": ["egedal", "1"],
    "Esbjerg": ["esbjerg", "1"],
    "Faxe": ["faxe", "1"],
    "Fredensborg": ["fredensborg", "1"],
    "Frederiksberg": ["frederiksberg", "1"],
    "Frederikssund": ["frederikssund", "1"],
    "Gentofte": ["gentofte", "1"],
    "Gladsaxe": ["gladsaxe", "1"],
    "Glostrup": ["glostrup", "1"],
    "Greve": ["greve", "1"],
    "Gribskov": ["gribskov", "1"],
    "Halsnæs": ["halsnaes", "1"],
    "Hedensted": ["hedensted", "1"],
    "Helsingør": ["helsingor", "1"],
    "Herlev": ["herlev", "1"],
    "Hillerød": ["hillerod", "1"],
    "Hjørring": ["hjoerring", "1"],
    "Horsens": ["horsens", "1"],
    "Hvidovre": ["hvidovre", "1"],
    "Høje-Taastrup": ["htk", "1"],
    "Hørsholm": ["hoersholm", "1"],
    "Jammerbugt": ["jammerbugt", "1"],
    "Kerteminde": ["kerteminde", "1"],
    "København": ["kk", "4"],
    "Køge": ["koege", "1"],
    "Lejre": ["lejre", "1"],
    "Lyngby-Taarbæk": ["ltf", "1"],
    "Mariagerfjord": ["mariagerfjord", "1"],
    "Næstved": ["naestved", "1"],
    "Odense": ["odense", "2"],
    "Odsherred": ["odsherred", "1"],
    "Randers": ["randers", "1"],
    "Ringkøbing-Skjern": ["rksk", "1"],
    "Ringsted": ["ringsted", "1"],
    "Roskilde": ["roskilde", "1"],
    "Rudersdal": ["rudersdal", "1"],
    "Rødovre": ["rk", "1"],
    "Samsø": ["samsoe", "1"],
    "Slagelse": ["slagelse", "1"],
    "Solrød": ["solrod", "1"],
    "Stevns": ["stevns", "1"],
    "Svendborg": ["svendborg", "1"],
    "Sønderborg": ["sonderborg", "1"],
    "Tårnby": ["taarnby", "1"],
    "Varde": ["varde", "1"],
    "Vejen": ["vejen", "1"],
    "Vordingborg": ["vordingborg", "1"],
}

MUNICIPALITIES_ARRAY = list(MUNICIPALITIES_LIST.keys())

ODD_EVEN_ARRAY = ["lige", "ulige"]

WEEKDAYS = ["Mandag", "Tirsdag", "Onsdag",
            "Torsdag", "Fredag", "Lørdag", "Søndag"]
WEEKDAYS_SHORT = ["Man", "Tir", "Ons", "Tor", "Fre", "Lør", "Søn"]
